import torch
# import torch.utils.data as data
# import torch.optim as optim
# from torch.autograd import Variable
import torch.nn as nn
import torch.nn.functional as F
# from copy import deepcopy

class VGG_miniimagenet(nn.Module):
    def __init__(self):
        super(VGG_miniimagenet,self).__init__()
        self.features = nn.Sequential(*[nn.Conv2d(3,64,kernel_size=3,padding=1),
                                       nn.ReLU(inplace=True),
                                       nn.MaxPool2d(kernel_size=2, stride=2),
                                       nn.Conv2d(64,128,kernel_size=3,padding=1),
                                       nn.ReLU(inplace=True),
                                       nn.MaxPool2d(kernel_size=2, stride=2),
                                       nn.Conv2d(128,256,kernel_size=3,padding=1),
                                       nn.ReLU(inplace=True),
                                       nn.Conv2d(256,256,kernel_size=3,padding=1),
                                       nn.ReLU(inplace=True),
                                       nn.MaxPool2d(kernel_size=2, stride=2)])
        self.convs = nn.Sequential(nn.Conv2d(256,128,kernel_size=3,padding=1),
                                   nn.BatchNorm2d(128, momentum=1.0),
                                   nn.ReLU(inplace=True),
                                   nn.Conv2d(128,128,kernel_size=3,padding=1),
                                   nn.BatchNorm2d(128, momentum=1.0),
                                   nn.ReLU(inplace=True),
                                   nn.MaxPool2d(kernel_size=2, stride=2))
        self.fc = nn.Sequential(nn.Linear(6*6*128,2))

    def forward(self, x):
        print(x.shape)
        x = self.features(x)
        x = self.convs(x)
        x = x.view(x.size(0), -1)
        x = self.fc(x)
        return x

class VGG_miniimagenet_r2(nn.Module):
    
    def __init__(self):
        super(VGG_miniimagenet_r2,self).__init__()
        self.features = nn.Sequential(*[nn.Conv2d(3,64,kernel_size=3,padding=1),
                                       nn.ReLU(inplace=True),
                                       nn.MaxPool2d(kernel_size=2, stride=2),
                                       nn.Conv2d(64,128,kernel_size=3,padding=1),
                                       nn.ReLU(inplace=True),
                                       nn.MaxPool2d(kernel_size=2, stride=2),
                                       nn.Conv2d(128,256,kernel_size=3,padding=1),
                                       nn.ReLU(inplace=True),
                                       nn.Conv2d(256,256,kernel_size=3,padding=1),
                                       nn.ReLU(inplace=True),
                                       nn.MaxPool2d(kernel_size=2, stride=2)])
        self.convs = nn.Sequential(nn.Conv2d(256,128,kernel_size=3,padding=1),
                                   nn.BatchNorm2d(128, momentum=1.0),
                                   nn.ReLU(inplace=True),
                                   nn.Conv2d(128,128,kernel_size=3,padding=1),
                                   nn.BatchNorm2d(128, momentum=1.0),
                                   nn.ReLU(inplace=True),
                                   nn.MaxPool2d(kernel_size=2, stride=2))
        self.fc = nn.Sequential(nn.Linear(6*6*128,2))
        self.resize = nn.Conv2d(128,256,kernel_size=1,padding=0)
        
    def _upsample_add(self, x, y):
        _,_,H,W = y.size()
        return F.interpolate(x, size=(H,W), mode='nearest') + y
    
    def forward(self, x):
        x = self.features(x)
        
        x_c = self.convs(x)
        x_r = self.resize(x_c)
        
        x_add = self._upsample_add(x_r,x)
        
        x = x_c.view(x.size(0), -1)
        x = self.fc(x)
        return x, x_add
    












    
